﻿using Ektron.Cms.Find.Application;
using Ektron.Cms.Find.Logging;
using Ektron.Cms.Find.Messaging;
using Ektron.Cms.Instrumentation;
using Ektron.Messaging;
using EPiServer.Find;
using System.Web;

namespace Ektron.Cms.Find.AspNetBootstrapper
{
    /// <summary>
    /// The EktronFindBootstrapper class is responsible for initializing and starting the
    /// systems necessary to index Ektron content with EPiServer Find.
    /// </summary>
    public static class EktronFindBootstrapper
    {
        /// <summary>
        /// Starts the systems necessary to index Ektron content with
        /// EPiServer Find. This method should be invoked when your
        /// application starts (e.g. in the Application_Start method of
        /// a Global.asax) to ensure that your content is indexed 
        /// appropriately.
        /// </summary>
        /// <remarks>
        /// This method may be edited to register custom type mappings for
        /// Smart Form definitions within your Ektron site. See the comments
        /// in the method source for an example of this registration.
        /// </remarks>
        public static void Start()
        {
            var applicationBuilder = new MessagingApplicationBuilder();
            var logger = new Logger(ObjectFactory.Get<ILogWriter>());
            var findClient = Client.CreateFromConfig();

            var findMessaging = new FindIndexingApplicationDirector();
            findMessaging.Logger = logger;
            findMessaging.FindClient = findClient;
            findMessaging.DocumentRepositoryFactory = (client) => new FindDocumentRepository(client);
            findMessaging.Configure(applicationBuilder);

            var ektronMessaging = new EktronIndexingApplicationDirector();
            ektronMessaging.ContentManagerFactory = () => ObjectFactory.GetContent();
            ektronMessaging.FolderManagerFactory = () => ObjectFactory.GetFolder();
            ektronMessaging.TaxonomyManagerFactory = () => ObjectFactory.GetTaxonomyManager();
            ektronMessaging.UserGroupManagerFactory = () => ObjectFactory.GetUserGroup();

            ektronMessaging.Logger = logger;
            ektronMessaging.FindClient = findClient;
            ektronMessaging.Configure(applicationBuilder);

            // Register type mappings below to translate Smart Forms 
            // to custom models. These mapped models will be indexed
            // within your Find index.
            
            // Example:
            //   ektronMessaging.Types.Register(content => new MySmartFormModel { ... })
            //                        .For(content => content.XmlConfiguration.Id == ...);

            applicationBuilder.App.Start();
        }
    }
}
